---
title: Deep Learning for Beginners
short_title: Beginners
description: Road map for beginners new to deep learning.
category: Get Started
weight: 10
---

## How Do I Start Using Deep Learning?

Where you start depends on what you already know. 

The prerequisites for really understanding deep learning are linear algebra, calculus and statistics, as well as programming and some machine learning. The prerequisites for applying it are just learning how to deploy a model. 

In the case of Deeplearning4j, you should know Java well and be comfortable with tools like the IntelliJ IDE and the automated build tool Maven. [Skymind's SKIL](https://docs.skymind.ai/) also includes a managed Conda environment for machine learning tools using Python. 

Below you'll find a list of resources. The sections are roughly organized in the order they will be useful. 

## Free Machine- and Deep-learning Courses Online

* [Andrew Ng's Machine-Learning Class on YouTube](https://www.youtube.com/watch?v=qeHZOdmJvFU) 
* [Geoff Hinton's Neural Networks Class on YouTube](https://youtu.be/2fRnHVVLf1Y) 
* [Patrick Winston's Introduction to Artificial Intelligence @MIT](http://ocw.mit.edu/courses/electrical-engineering-and-computer-science/6-034-artificial-intelligence-fall-2010/) (For those interested in a survey of artificial intelligence.)
* [Andrej Karpathy's Convolutional Neural Networks Class at Stanford](http://cs231n.github.io) (For those interested in image recognition.)
* [ML@B: Machine Learning Crash Course: Part 1](https://ml.berkeley.edu/blog/2016/11/06/tutorial-1/)
* [ML@B: Machine Learning Crash Course: Part 2](https://ml.berkeley.edu/blog/2016/12/24/tutorial-2/)
* [Gradient descent, how neural networks learn, Deep learning, part 2](https://www.youtube.com/watch?v=IHZwWFHWa-w&feature=youtu.be)

## Math

The math involved with deep learning is basically linear algebra, calculus and probility, and if you have studied those at the undergraduate level, you will be able to understand most of the ideas and notation in deep-learning papers. If haven't studied those in college, never fear. There are many free resources available (and some on this website).

* [Calculus Made Easy, by Silvanus P. Thompson](http://www.gutenberg.org/ebooks/33283?msg=welcome_stranger)
* [Seeing Theory: A Visual Introduction to Probability and Statistics](http://students.brown.edu/seeing-theory/)
* [Andrew Ng's 6-Part Review of Linear Algebra](https://www.youtube.com/playlist?list=PLnnr1O8OWc6boN4WHeuisJWmeQHH9D_Vg)
* [Khan Academy's Linear Algebra Course](https://www.khanacademy.org/math/linear-algebra)
* [Linear Algebra for Machine Learning](https://www.youtube.com/watch?v=ZumgfOei0Ak); Patrick van der Smagt
* [CMU's Linear Algebra Review](http://www.cs.cmu.edu/~zkolter/course/linalg/outline.html)
* [Math for Machine Learning](https://www.umiacs.umd.edu/~hal/courses/2013S_ML/math4ml.pdf)
* [Immersive Linear Algebra](http://immersivemath.com/ila/learnmore.html)
* [Probability Cheatsheet](https://static1.squarespace.com/static/54bf3241e4b0f0d81bf7ff36/t/55e9494fe4b011aed10e48e5/1441352015658/probability_cheatsheet.pdf)
* [The best linear algebra books](https://begriffs.com/posts/2016-07-24-best-linear-algebra-books.html)
* [Markov Chains, Visually Explained](http://setosa.io/ev/markov-chains/)
* [An Introduction to MCMC for Machine Learning](http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.13.7133&rep=rep1&type=pdf)
* [Eigenvectors, Eigenvalues, PCA, Covariance and Entropy](https://skymind.ai/wiki/eigenvector)
* [Markov Chain Monte Carlo (MCMC) & Machine Learning](https://skymind.ai/wiki/markov-chain-monte-carlo)
* [Relearning Matrices as Linear Functions](https://www.dhruvonmath.com/2018/12/31/matrices/)

## Programming

If you do not know how to program yet, you can start with Java, but you might find other languages easier. Python and Ruby resources can convey the basic ideas in a faster feedback loop. "Learn Python the Hard Way" and "Learn to Program (Ruby)" are two great places to start. 

* [Scratch: A Visual Programming Environment From MIT](https://scratch.mit.edu/)
* [Learn to Program (Ruby)](https://pine.fm/LearnToProgram/)
* [Grasshopper: A Mobile App to Learn Basic Coding (Javascript)](https://grasshopper.codes/)
* [Intro to the Command Line](http://cli.learncodethehardway.org/book/)
* [Additional command-line tutorial](http://www.learnenough.com/command-line)
* [A Vim Tutorial and Primer](https://danielmiessler.com/study/vim/) (Vim is an editor accessible from the command line.)
* [Intro to Computer Science (CS50 @Harvard edX)](https://www.edx.org/course/introduction-computer-science-harvardx-cs50x)
* [A Gentle Introduction to Machine Fundamentals](https://marijnhaverbeke.nl/turtle/)
* [Teaching C](https://blog.regehr.org/archives/1393)

If you want to jump into deep-learning from here without Java, we recommend [Theano](http://deeplearning.net/) and the various Python frameworks built atop it, including [Keras](https://github.com/fchollet/keras) and [Lasagne](https://github.com/Lasagne/Lasagne).

## Python

* [Learn Python the Hard Way](http://learnpythonthehardway.org/)
* [Google's Python Class](https://developers.google.com/edu/python/)
* [Udemy: Complete Python 3 Masterclass Journey](https://www.udemy.com/complete-python-3-masterclass-journey/)
* [MIT: Introduction to Computer Science and Python Programming](https://ocw.mit.edu/courses/electrical-engineering-and-computer-science/6-0001-introduction-to-computer-science-and-programming-in-python-fall-2016/) 
* [David Beazley: Python Tutorials](http://www.dabeaz.com/tutorials.html)
* [CS231n: Python Numpy Tutorial](http://cs231n.github.io/python-numpy-tutorial/)
* [Pyret: A Python Learning Environment](https://www.pyret.org/)

## Java

Once you have programming basics down, tackle Java, the world's most widely used programming language. Most large organizations in the world operate on huge Java code bases. (There will always be Java jobs.) The big data stack -- Hadoop, Spark, Kafka, Lucene, Solr, Cassandra, Flink -- have largely been written for Java's compute environment, the JVM.

* [Think Java: Interactive Web-based Dev Environment](https://books.trinket.io/thinkjava/)
* [Learn Java The Hard Way](https://learnjavathehardway.org/)
* [Introduction to JShell](https://docs.oracle.com/javase/10/jshell/introduction-jshell.htm#JSHEL-GUID-630F27C8-1195-4989-9F6B-2C51D46F52C8)
* [JShell in 5 Minutes](https://dzone.com/articles/jshell-in-five-minutes)
* [Java Resources](http://wiht.link/java-resources)
* [Java Ranch: A Community for Java Beginners](http://javaranch.com/)
* [Intro to Programming in Java @Princeton](http://introcs.cs.princeton.edu/java/home/)
* [Head First Java](http://www.amazon.com/gp/product/0596009208)
* [Java in a Nutshell](http://www.amazon.com/gp/product/1449370829)
* [Java Programming for Complete Beginners in 250 Steps](https://www.udemy.com/java-tutorial/)

## Deeplearning4j

With that under your belt, we recommend you approach Deeplearning4j through its [examples](https://github.com/eclipse/deeplearning4j-examples). 

* [Quickstart](./deeplearning4j-quickstart)

You can also download a [free version of the Skymind Intelligence Layer](https://docs.skymind.ai/), which supports Python, Java and Scala machine-learning and data science tools. SKIL is a machine-learning backend that works on prem and in the cloud, and can ship with your software to provide a machine learning model server. 

## Other Resources

Most of what we know about deep learning is contained in academic papers. You can find some of the major research groups [here](https://skymind.ai/wiki/machine-learning-research-groups-labs).

While individual courses have limits on what they can teach, the Internet does not. Most math and programming questions can be answered by Googling and searching sites like [Stackoverflow](http://stackoverflow.com) and [Math Stackexchange](https://math.stackexchange.com/).
